
/*
 * Copyright (c) 2023 The DragonFly Project.  All rights reserved.
 *
 * This code is derived from software contributed to The DragonFly Project
 * by Matthew Dillon <dillon@backplane.com>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 3. Neither the name of The DragonFly Project nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific, prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
/*
 *
 *
 */
#include <sys/types.h>
#include <sys/time.h>
#include <sys/wait.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>

int
main(int ac, char **av)
{
	int i;
	int n;
	int ttl;
	struct timeval tv1;
	struct timeval tv2;
	int fds[2];
	int ignore_first = 1;

	if (ac == 2 && av[1][0] != 'x') {
		char buf[65536];
		int base_secs = 1;

		pipe(fds);
		dup2(fds[1], 5);
		n = strtol(av[1], NULL, 0);
		for (i = 0; i < n; ++i) {
			char *buf;

			asprintf(&buf, "cp %s %s.%d", av[0], av[0], i);
			system(buf);
			free(buf);
			if (fork() == 0) {
				close(fds[0]);
				close(fds[1]);
				asprintf(&buf, "%s.%d", av[0], i);
				/* asprintf(&buf, "%s", av[0]); */
				execl(buf, buf, "x1", NULL);
				_exit(1);
			}
		}
		gettimeofday(&tv1, NULL);
		ttl = 0;
		for (;;) {
			long usecs;

			n = read(fds[0], buf, sizeof(buf));
			if (n < 0)
				break;
			gettimeofday(&tv2, NULL);
			ttl += n * 16;
			usecs = (tv2.tv_sec - tv1.tv_sec) * 1000000L;
			usecs += (long)(tv2.tv_usec - tv1.tv_usec);
			if (usecs >= (double)base_secs * 1e6) {
				if (!ignore_first) {
					printf("%6.2f/sec\n",
						(double)ttl /
						 ((double)usecs / 1e6));
				}
				ignore_first = 0;
				gettimeofday(&tv1, NULL);
				ttl = 0;
			}
			usleep(10000);
		}
		/*
		while (wait3(&status, 0, NULL) >= 0 || errno == EINTR)
			;
		*/
		exit(0);
	}
	if (ac == 2) {
		char buf[64];
		long n;

		n = strtoul(av[1] + 1, NULL, 0);

		snprintf(buf, sizeof(buf), "x%ld", n + 1);
		if ((n & 15) == 0)
		    write(5, "x", 1);
		execl(av[0], av[0], buf, NULL);
	}

	return 0;
}
